import React, {
  ReactNode,
  createContext,
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import useSWR from "swr";
import CircularProgress from "./components/circularProgress";
import useLocalStorage from "./hooks/useLocalStorage";

interface LocaleContextType {
  locale: string;
  handleLocaleChange: (newLocale: string) => void;
}

export const LocaleContext = createContext<LocaleContextType>({
  locale: "",
  handleLocaleChange: Function,
});

async function fetcher(path: RequestInfo | URL) {
  const res = await fetch(path);
  const json = await res.json();
  return json;
}

export const LocaleProvider = ({
  geoApiUrl = "/api/geo",
  children,
}: {
  geoApiUrl?: string;
  children: ReactNode;
}) => {
  const { data, isValidating } = useSWR(geoApiUrl, fetcher);
  const [languageStored, setLanguageStored] = useLocalStorage(
    "language",
    undefined
  );
  const [locale, setLocale] = useState<string>(languageStored || "");

  useEffect(() => {
    if (!languageStored && data?.geo) {
      handleLocaleChange(data?.geo);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [data?.geo]);

  useEffect(() => {
    handleLocaleChange(locale);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [locale]);

  const handleLocaleChange = useCallback(
    (newLocale: string): void => {
      setLocale(newLocale);
      setLanguageStored(newLocale as never);
    },
    [setLanguageStored]
  );

  const localeContextValue = useMemo(
    () => ({ locale, handleLocaleChange }),
    [locale, handleLocaleChange]
  );

  if (isValidating) {
    return <CircularProgress />;
  }

  return (
    <LocaleContext.Provider value={localeContextValue}>
      {children}
    </LocaleContext.Provider>
  );
};

export const useLocaleContext = () => {
  const context = useContext(LocaleContext);
  if (!context) {
    throw new Error("Missing locale context");
  }

  return context;
};
